@@ -1,12 +1,12 @@
 /*
- * Copyright (c) 2008 Internet Initiative Japan Inc. All rights reserved.
+ * Copyright (c) 2008-2009 Internet Initiative Japan Inc. All rights reserved.
  *
  * The terms and conditions of the accompanying program
  * shall be provided separately by Internet Initiative Japan Inc.
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: daemonize.h 139 2008-06-27 08:55:53Z tsuruda $
+ * $Id: daemonize.h 579 2009-01-12 16:19:17Z takahiko $
  */
 
 #ifndef __DAEMONIZE_H__
@@ -14,8 +14,7 @@
 
 #include <stdbool.h>
 
-bool daemonize_init(const char *username, const char *chdirpath, const char *pidfile,
-                    int argc, char **argv);
+bool daemonize_init(const char *username, const char *chdirpath, const char *pidfile);
 bool daemonize_finally(const char *pidfile);
 
 #endif
