@@ -1,12 +1,12 @@
 /*
- * Copyright (c) 2008 Internet Initiative Japan Inc. All rights reserved.
+ * Copyright (c) 2008-2009 Internet Initiative Japan Inc. All rights reserved.
  *
  * The terms and conditions of the accompanying program
  * shall be provided separately by Internet Initiative Japan Inc.
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id:enma_config.h 154 2008-07-07 08:16:11Z tsuruda $
+ * $Id: enma_config.h 875 2009-04-01 07:33:04Z tsuruda $
  */
 
 #ifndef __ENMA_CONFIG_H__
@@ -16,6 +16,8 @@
 #include <assert.h>
 #include <stdbool.h>
 
+#include "ipaddressrange.h"
+
 // 設定情報の記憶領域
 typedef struct EnmaConfig {
     // milter
@@ -27,16 +29,24 @@ typedef struct EnmaConfig {
     const char *milter_chdir;
     int milter_timeout;
     int milter_loglevel;
+    int milter_sendmail813;     //boolean
     int milter_postfix;         //boolean
     // syslog
     const char *syslog_ident;
     int syslog_facility;
     int syslog_logmask;
-    // authresult
+    // common
+    IPAddressRangeList *common_exclusion_addresses;
+    // sender authentication
     int spf_auth;               //boolean
     int spf_explog;             //boolean
     int sidf_auth;              //boolean
     int sidf_explog;            //boolean
+    int dkim_auth;              //boolean
+    int dkim_signheader_limit;
+    int dkim_accept_expired_signature;  //boolean
+    int dkimadsp_auth;          //boolean
+    // authentication-results
     const char *authresult_identifier;
 } EnmaConfig;
 
