@@ -1,12 +1,12 @@
 /*
- * Copyright (c) 2008 Internet Initiative Japan Inc. All rights reserved.
+ * Copyright (c) 2008-2009 Internet Initiative Japan Inc. All rights reserved.
  *
  * The terms and conditions of the accompanying program
  * shall be provided separately by Internet Initiative Japan Inc.
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: enma_mfi.h 154 2008-07-07 08:16:11Z tsuruda $
+ * $Id: enma_mfi.h 654 2009-02-28 20:46:58Z takahiko $
  */
 
 #ifndef __ENMA_MFI_H__
@@ -21,6 +21,8 @@ extern sfsistat mfi_helo(SMFICTX *ctx, char *helohost);
 extern sfsistat mfi_envfrom(SMFICTX *ctx, char **argv);
 extern sfsistat mfi_envrcpt(SMFICTX *ctx, char **argv);
 extern sfsistat mfi_header(SMFICTX *ctx, char *headerf, char *headerv);
+extern sfsistat mfi_eoh(SMFICTX *ctx);
+extern sfsistat mfi_body(SMFICTX *ctx, unsigned char *bodyp, size_t bodylen);
 extern sfsistat mfi_eom(SMFICTX *ctx);
 extern sfsistat mfi_abort(SMFICTX *ctx);
 extern sfsistat mfi_close(SMFICTX *ctx);
