@@ -1,12 +1,12 @@
 /*
- * Copyright (c) 2008 Internet Initiative Japan Inc. All rights reserved.
+ * Copyright (c) 2008-2009 Internet Initiative Japan Inc. All rights reserved.
  *
  * The terms and conditions of the accompanying program
  * shall be provided separately by Internet Initiative Japan Inc.
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: enma_mfi_ctx.h 193 2008-07-17 04:34:28Z tsuruda $
+ * $Id: enma_mfi_ctx.h 655 2009-02-28 20:47:41Z takahiko $
  */
 
 #ifndef __ENMA_MFI_CTX_H__
@@ -21,7 +21,7 @@
 #include "mailheaders.h"
 #include "dnsresolv.h"
 #include "sidf.h"
-#include "sidfrequest.h"
+#include "dkim.h"
 #include "authresult.h"
 
 typedef struct EnmaMfiCtx {
@@ -36,6 +36,7 @@ typedef struct EnmaMfiCtx {
     char *qid;
     InetMailbox *envfrom;
     MailHeaders *headers;
+    DkimVerifySession *dkimverifier;
     AuthResult *authresult;
     // Authentication-Results ヘッダを削るためのメンバ
     int authhdr_count;          // 遭遇した Authentication-Results ヘッダの数
