@@ -1,12 +1,12 @@
 /*
- * Copyright (c) 2008 Internet Initiative Japan Inc. All rights reserved.
+ * Copyright (c) 2008-2009 Internet Initiative Japan Inc. All rights reserved.
  *
  * The terms and conditions of the accompanying program
  * shall be provided separately by Internet Initiative Japan Inc.
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id:enma_config.h 154 2008-07-07 08:16:11Z tsuruda $
+ * $Id: enma_sidf.h 787 2009-03-25 01:09:55Z takahiko $
  */
 
 #ifndef __ENMA_SIDF_H__
@@ -18,7 +18,7 @@
 #include "inetmailbox.h"
 #include "mailheaders.h"
 #include "dnsresolv.h"
-#include "sidfpolicy.h"
+#include "sidf.h"
 #include "authresult.h"
 
 extern bool EnmaSpf_evaluate(SidfPolicy *policy, DnsResolver *resolver, AuthResult *authresult,
