@@ -1,12 +1,12 @@
 /*
- * Copyright (c) 2008 Internet Initiative Japan Inc. All rights reserved.
+ * Copyright (c) 2008-2009 Internet Initiative Japan Inc. All rights reserved.
  *
  * The terms and conditions of the accompanying program
  * shall be provided separately by Internet Initiative Japan Inc.
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: string_util.h 91 2008-06-23 07:38:37Z tsuruda $
+ * $Id: string_util.h 828 2009-03-27 03:20:12Z tsuruda $
  */
 
 #ifndef __STRING_UTIL_H__
@@ -14,7 +14,7 @@
 
 #include <stdbool.h>
 
-extern bool isdigits(const char *string);
+extern long int strtolstrict(const char *string, bool *errflag);
 extern char *strlstrip(char *string);
 extern char *strrstrip(char *string);
 extern char *strstrip(char *string);
