@@ -1,16 +1,16 @@
 /*
- * Copyright (c) 2008 Internet Initiative Japan Inc. All rights reserved.
+ * Copyright (c) 2008-2009 Internet Initiative Japan Inc. All rights reserved.
  *
  * The terms and conditions of the accompanying program
  * shall be provided separately by Internet Initiative Japan Inc.
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: config_loader.c 479 2008-08-25 11:46:35Z tsuruda $
+ * $Id: config_loader.c 830 2009-03-27 04:44:28Z tsuruda $
  */
 
 #include "rcsid.h"
-RCSID("$Id: config_loader.c 479 2008-08-25 11:46:35Z tsuruda $");
+RCSID("$Id: config_loader.c 830 2009-03-27 04:44:28Z tsuruda $");
 
 #include <stdio.h>
 #include <assert.h>
@@ -21,15 +21,20 @@ RCSID("$Id: config_loader.c 479 2008-08-25 11:46:35Z tsuruda $");
 #include <strings.h>
 #include <stddef.h>
 #include <syslog.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 
 #include "ptrop.h"
+#include "strarray.h"
 
+#include "ipaddressrange.h"
 #include "consolehandler.h"
 #include "string_util.h"
 #include "syslogtable.h"
 #include "config_loader.h"
 
-
 /**
  * 渡された文字列情報を、文字列のまま設定の保存領域に記憶
  *
@@ -53,7 +58,6 @@ ConfigLoader_setString(void *config_storage, const char *config_value)
     return true;
 }
 
-
 /**
  * Convert from string to bool
  *
@@ -90,7 +94,6 @@ ConfigLoader_setBoolean(void *config_storage, const char *config_value)
     return false;
 }
 
-
 /**
  * 渡された文字列情報を、int に変換し設定の保存領域に記憶
  *
@@ -104,16 +107,17 @@ ConfigLoader_setInteger(void *config_storage, const char *config_value)
     assert(NULL != config_storage);
     assert(NULL != config_value);
 
-    if (!isdigits(config_value)) {
-        // 形式が数字ではない
+    int *config_int_p = (int *) config_storage;
+
+    bool errflag;
+    long int parsed_long = strtolstrict(config_value, &errflag);
+    if (errflag) {
         return false;
     }
-    int *config_int_p = (int *) config_storage;
-    *config_int_p = atoi(config_value);
+    *config_int_p = (int) parsed_long;
     return true;
 }
 
-
 /**
  * 渡された文字列情報を、long int に変換し設定の保存領域に記憶
  *
@@ -127,16 +131,17 @@ ConfigLoader_setLong(void *config_storage, const char *config_value)
     assert(NULL != config_storage);
     assert(NULL != config_value);
 
-    if (!isdigits(config_value)) {
-        // 形式が数字ではない
+    long int *config_long_p = (long int *) config_storage;
+
+    bool errflag;
+    long int parsed_long = strtolstrict(config_value, &errflag);
+    if (errflag) {
         return false;
     }
-    long int *config_long_p = (long int *) config_storage;
-    *config_long_p = atol(config_value);
+    *config_long_p = parsed_long;
     return true;
 }
 
-
 /**
  * 渡された文字列情報を、syslogのfacilyを示すint型に変換し設定の保存領域に記憶
  *
@@ -152,14 +157,13 @@ ConfigLoader_setSyslogFacility(void *config_storage, const char *config_value)
 
     int *config_int_p = (int *) config_storage;
 
-    if (-1 == (*config_int_p = lookup_facility_const(config_value))) {;
+    if (-1 == (*config_int_p = lookup_facility_const(config_value))) {
         return false;
     }
 
     return true;
 }
 
-
 /**
  * 渡された文字列情報を、syslogのpriorityを示すint型に変換し設定の保存領域に記憶
  *
@@ -175,13 +179,101 @@ ConfigLoader_setSyslogPriority(void *config_storage, const char *config_value)
 
     int *config_int_p = (int *) config_storage;
 
-    if (-1 == (*config_int_p = lookup_priority_const(config_value))) {;
+    if (-1 == (*config_int_p = lookup_priority_const(config_value))) {
         return false;
     }
 
     return true;
 }
 
+/**
+ * 渡された文字列情報を、IPAddressRange に格納
+ *
+ * @param config_storage
+ * @param config_value
+ * @return
+ */
+static bool
+ConfigLoader_setIPAddress(void *config_storage, const char *config_value)
+{
+    assert(NULL != config_storage);
+    assert(NULL != config_value);
+
+    IPAddressRange **config_ip_address_p = (IPAddressRange **) config_storage;
+
+    *config_ip_address_p = IPAddressRange_new();
+    if (NULL == *config_ip_address_p) {
+        ConsoleError("memory allocation failed: error=%s", strerror(errno));
+        return false;
+    }
+
+    if (!IPAddressRange_set(*config_ip_address_p, config_value)) {
+        ConsoleError("failed to parse address: address=%s", config_value);
+        goto error;
+    }
+
+    return true;
+
+  error:
+    if (NULL != *config_ip_address_p) {
+        IPAddressRange_free(*config_ip_address_p);
+        *config_ip_address_p = NULL;
+    }
+    return false;
+}
+
+/**
+ * 渡された文字列情報を、IPAddressRangeList に格納
+ *
+ * @param config_storage
+ * @param config_value
+ * @return
+ */
+static bool
+ConfigLoader_setIPAddressList(void *config_storage, const char *config_value)
+{
+    assert(NULL != config_storage);
+    assert(NULL != config_value);
+
+    IPAddressRangeList **config_ip_address_list_p = (IPAddressRangeList **) config_storage;
+
+    /* 渡されたIPアドレス情報を、特定の文字で分割 */
+    StrArray *addr_array = StrArray_split(config_value, ", ", true);
+    if (addr_array == NULL) {
+        ConsoleError("memory allocation failed: error=%s", strerror(errno));
+        return false;
+    }
+
+    /* IPアドレスのリストを作成 */
+    int range_list_len = (int) StrArray_getCount(addr_array);
+    *config_ip_address_list_p = IPAddressRangeList_new(range_list_len);
+    if (NULL == *config_ip_address_list_p) {
+        ConsoleError("memory allocation failed: error=%s", strerror(errno));
+        goto error;
+    }
+
+    const char *addr;
+    for (int i = 0; i < range_list_len; ++i) {
+        addr = StrArray_get(addr_array, i);
+        if (!IPAddressRangeList_set(*config_ip_address_list_p, (size_t) i, addr)) {
+            ConsoleError("failed to parse address: address=%s", addr);
+            goto error;
+        }
+    }
+    StrArray_free(addr_array);
+
+    return true;
+
+  error:
+    if (NULL != *config_ip_address_list_p) {
+        IPAddressRangeList_free(*config_ip_address_list_p);
+        *config_ip_address_list_p = NULL;
+    }
+    if (NULL != addr_array) {
+        StrArray_free(addr_array);
+    }
+    return false;
+}
 
 /**
  * 既に設定情報が保持されているかを判定する
@@ -196,9 +288,11 @@ ConfigLoader_isSet(void *config_storage, const ConfigType config_type)
     assert(NULL != config_storage);
 
     switch (config_type) {
-    case CONFIGTYPE_STRING:;
-        char **config_char_p = (char **) config_storage;
-        if (NULL == *config_char_p) {
+    case CONFIGTYPE_STRING:
+    case CONFIGTYPE_IP_ADDRESS:
+    case CONFIGTYPE_IP_ADDRESS_LIST:;
+        void **config_ptr = (void **) config_storage;
+        if (NULL == *config_ptr) {
             return false;
         }
         break;
@@ -220,7 +314,6 @@ ConfigLoader_isSet(void *config_storage, const ConfigType config_type)
     return true;
 }
 
-
 /**
  * 各設定情報の型に合わせて設定情報を記憶
  *
@@ -230,8 +323,8 @@ ConfigLoader_isSet(void *config_storage, const ConfigType config_type)
  * @return
  */
 static bool
-ConfigLoader_setValue(const ConfigEntry *config_entry, void *config_struct,
-                      const char *config_value)
+ConfigLoader_setValue(const ConfigEntry *config_entry,
+                      void *config_struct, const char *config_value)
 {
     assert(NULL != config_entry);
     assert(NULL != config_struct);
@@ -274,6 +367,16 @@ ConfigLoader_setValue(const ConfigEntry *config_entry, void *config_struct,
             return false;
         }
         break;
+    case CONFIGTYPE_IP_ADDRESS:
+        if (!ConfigLoader_setIPAddress(config_storage, config_value)) {
+            return false;
+        }
+        break;
+    case CONFIGTYPE_IP_ADDRESS_LIST:
+        if (!ConfigLoader_setIPAddressList(config_storage, config_value)) {
+            return false;
+        }
+        break;
     default:
         ConsoleError("unknown config type: type=%d", config_entry->config_type);
         abort();
@@ -281,7 +384,6 @@ ConfigLoader_setValue(const ConfigEntry *config_entry, void *config_struct,
     return true;
 }
 
-
 /**
  * 指定された設定項目に対応するエントリを返す
  *
@@ -303,7 +405,6 @@ ConfigLoader_lookupEntry(const ConfigEntry *config_entry, const char *entry_name
     return NULL;
 }
 
-
 /**
  * 指定されたファイルから設定情報を読み込み、記憶する
  *
@@ -313,8 +414,8 @@ ConfigLoader_lookupEntry(const ConfigEntry *config_entry, const char *entry_name
  * @return
  */
 bool
-ConfigLoader_setConfigValue(const ConfigEntry *config_entry, const char *filename,
-                            void *config_struct)
+ConfigLoader_setConfigValue(const ConfigEntry *config_entry,
+                            const char *filename, void *config_struct)
 {
     assert(NULL != config_entry);
     assert(NULL != filename);
@@ -349,7 +450,8 @@ ConfigLoader_setConfigValue(const ConfigEntry *config_entry, const char *filenam
         (void) strstrip(config_key);
         (void) strstrip(config_value);
 
-        const ConfigEntry *entry = ConfigLoader_lookupEntry(config_entry, config_key);
+        const ConfigEntry *entry = ConfigLoader_lookupEntry(config_entry,
+                                                            config_key);
         if (NULL == entry) {
             ConsoleNotice("config parse failed: file=%s:%d, key=%s, value=%s", filename,
                           current_line_no, config_key, config_value);
@@ -382,7 +484,6 @@ ConfigLoader_setConfigValue(const ConfigEntry *config_entry, const char *filenam
     return false;
 }
 
-
 /**
  * デフォルトの設定情報を設定の保存領域に記憶
  *
@@ -407,7 +508,6 @@ ConfigLoader_setDefaultValue(const ConfigEntry *config_entry, void *config_struc
     return true;
 }
 
-
 /**
  * オプションとして渡された引数の設定情報を保存領域に記憶
  *
@@ -418,15 +518,16 @@ ConfigLoader_setDefaultValue(const ConfigEntry *config_entry, void *config_struc
  * @return
  */
 bool
-ConfigLoader_setOptionValue(const ConfigEntry *config_entry, const char *config_key,
-                            const char *config_value, void *config_struct)
+ConfigLoader_setOptionValue(const ConfigEntry *config_entry,
+                            const char *config_key, const char *config_value, void *config_struct)
 {
     assert(NULL != config_entry);
     assert(NULL != config_key);
     assert(NULL != config_value);
     assert(NULL != config_struct);
 
-    const ConfigEntry *entry = ConfigLoader_lookupEntry(config_entry, config_key);
+    const ConfigEntry *entry = ConfigLoader_lookupEntry(config_entry,
+                                                        config_key);
     if (NULL == entry) {
         ConsoleNotice("config parse failed: key=%s, value=%s", config_key, config_value);
         return false;
@@ -440,7 +541,6 @@ ConfigLoader_setOptionValue(const ConfigEntry *config_entry, const char *config_
     return true;
 }
 
-
 /**
  * -oオプションで渡された'='付きの設定情報を保存領域に記憶
  *
@@ -450,8 +550,8 @@ ConfigLoader_setOptionValue(const ConfigEntry *config_entry, const char *config_
  * @return
  */
 bool
-ConfigLoader_setEqualStringOptionValue(const ConfigEntry *config_entry, const char *optarg,
-                                       void *config_struct)
+ConfigLoader_setEqualStringOptionValue(const ConfigEntry *config_entry,
+                                       const char *optarg, void *config_struct)
 {
     assert(NULL != config_entry);
     assert(NULL != optarg);
@@ -488,7 +588,6 @@ ConfigLoader_setEqualStringOptionValue(const ConfigEntry *config_entry, const ch
     return false;
 }
 
-
 /**
  * 設定情報の記憶域の確保
  *
@@ -516,6 +615,16 @@ ConfigLoader_init(const ConfigEntry *config_entry, void *config_struct)
             int *config_int_p = (int *) STRUCT_MEMBER_P(config_struct, p->struct_offset);
             *config_int_p = -1;
             break;
+        case CONFIGTYPE_IP_ADDRESS:;
+            IPAddressRange **config_ip_address_p =
+                (IPAddressRange **) STRUCT_MEMBER_P(config_struct, p->struct_offset);
+            *config_ip_address_p = NULL;
+            break;
+        case CONFIGTYPE_IP_ADDRESS_LIST:;
+            IPAddressRangeList **config_ip_address_list_p =
+                (IPAddressRangeList **) STRUCT_MEMBER_P(config_struct, p->struct_offset);
+            *config_ip_address_list_p = NULL;
+            break;
         default:
             ConsoleError("unknown config type: type=%d", p->config_type);
             abort();
@@ -523,7 +632,6 @@ ConfigLoader_init(const ConfigEntry *config_entry, void *config_struct)
     }
 }
 
-
 /**
  * 設定情報の記憶領域を解放
  *
@@ -549,6 +657,22 @@ ConfigLoader_free(const ConfigEntry *config_entry, void *config_struct)
         case CONFIGTYPE_SYSLOG_FACILITY:
         case CONFIGTYPE_SYSLOG_PRIORITY:
             break;
+        case CONFIGTYPE_IP_ADDRESS:;
+            IPAddressRange **config_ipaddress_p =
+                (IPAddressRange **) STRUCT_MEMBER_P(config_struct, p->struct_offset);
+            if (NULL != *config_ipaddress_p) {
+                IPAddressRange_free(*config_ipaddress_p);
+                *config_ipaddress_p = NULL;
+            }
+            break;
+        case CONFIGTYPE_IP_ADDRESS_LIST:;
+            IPAddressRangeList **config_ipaddress_list_p =
+                (IPAddressRangeList **) STRUCT_MEMBER_P(config_struct, p->struct_offset);
+            if (NULL != *config_ipaddress_list_p) {
+                IPAddressRangeList_free(*config_ipaddress_list_p);
+                *config_ipaddress_list_p = NULL;
+            }
+            break;
         default:
             ConsoleError("unknown config type: type=%d", p->config_type);
             abort();
@@ -556,7 +680,6 @@ ConfigLoader_free(const ConfigEntry *config_entry, void *config_struct)
     }
 }
 
-
 /**
  * 現在の設定情報を表示する
  *
@@ -576,6 +699,7 @@ ConfigLoader_dump(const ConfigEntry *config_entry, const void *config_struct, FI
         fprintf(out, "  %s: ", p->config_name);
         void *value = STRUCT_MEMBER_P(config_struct, p->struct_offset);
 
+        int retval;
         switch (p->config_type) {
         case CONFIGTYPE_STRING:
             fprintf(out, "%s", NNSTR(*(char **) value));
@@ -595,6 +719,29 @@ ConfigLoader_dump(const ConfigEntry *config_entry, const void *config_struct, FI
         case CONFIGTYPE_SYSLOG_PRIORITY:
             fprintf(out, "%s", lookup_priority_name(*(int *) value));
             break;
+        case CONFIGTYPE_IP_ADDRESS:;
+            IPAddressRange **addr_range_p = (IPAddressRange **) value;
+            char addrbuf[INET6_ADDRSTRLEN + sizeof("/128")];
+            retval = IPAddressRange_toString(*addr_range_p, addrbuf, sizeof(addrbuf));
+            if (retval < 0) {
+                ConsoleError("IPAddressRange_toString failed: error=%s", strerror(retval));
+                abort();
+            }
+            fprintf(out, "[%s]", addrbuf);
+            break;
+        case CONFIGTYPE_IP_ADDRESS_LIST:;
+            IPAddressRangeList **addr_range_list_p = (IPAddressRangeList **) value;
+            char rangebuf[INET6_ADDRSTRLEN + sizeof("/128")];
+            for (int i = 0; i < (int) IPAddressRangeList_getCount(*addr_range_list_p); ++i) {
+                const IPAddressRange *addr_range = IPAddressRangeList_get(*addr_range_list_p, i);
+                retval = IPAddressRange_toString(addr_range, rangebuf, sizeof(rangebuf));
+                if (retval < 0) {
+                    ConsoleError("IPAddressRange_toString failed: error=%s", strerror(retval));
+                    abort();
+                }
+                fprintf(out, "[%s]", rangebuf);
+            }
+            break;
         default:
             ConsoleError("unknown config type: type=%d", p->config_type);
             abort();
