@@ -1,16 +1,16 @@
 /*
- * Copyright (c) 2008 Internet Initiative Japan Inc. All rights reserved.
+ * Copyright (c) 2008-2009 Internet Initiative Japan Inc. All rights reserved.
  *
  * The terms and conditions of the accompanying program
  * shall be provided separately by Internet Initiative Japan Inc.
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id$
+ * $Id: enma_mfi.c 875 2009-04-01 07:33:04Z tsuruda $
  */
 
 #include "rcsid.h"
-RCSID("$Id$");
+RCSID("$Id: enma_mfi.c 875 2009-04-01 07:33:04Z tsuruda $");
 
 #include <assert.h>
 #include <errno.h>
@@ -21,16 +21,15 @@ RCSID("$Id$");
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
+#include <openssl/err.h>
 #include <libmilter/mfapi.h>
 
 #include "ptrop.h"
 #include "loghandler.h"
+#include "bitmemcmp.h"
 #include "dnsresolv.h"
 #include "sidf.h"
-#include "sidfpra.h"
-#include "sidfpolicy.h"
-#include "sidfrequest.h"
-#include "sidfenum.h"
+#include "dkim.h"
 #include "mailheaders.h"
 #include "intarray.h"
 #include "xskip.h"
@@ -40,7 +39,10 @@ RCSID("$Id$");
 #include "enma_mfi.h"
 #include "enma_config.h"
 #include "enma_sidf.h"
+#include "enma_dkim.h"
 #include "enma_mfi_ctx.h"
+#include "ipaddressrange.h"
+#include "addr_util.h"
 
 #define UNKNOWN_HOSTNAME "(unknown)"
 #define UNKNOWN_QID "(unknown)"
@@ -54,8 +56,8 @@ struct smfiDesc mfi_desc = {
     mfi_envfrom,    /* envelope sender filter */
     NULL,   /* envelope recipient filter */
     mfi_header, /* header filter */
-    NULL,   /* end of header */
-    NULL,   /* body block filter */
+    mfi_eoh,    /* end of header */
+    mfi_body,   /* body block filter */
     mfi_eom,    /* end of message */
     mfi_abort,  /* message aborted */
     mfi_close,  /* connection cleanup */
@@ -151,53 +153,12 @@ static sfsistat
 EnmaMfi_tempfail(EnmaMfiCtx *enma_mfi_ctx)
 {
     EnmaMfiCtx_reset(enma_mfi_ctx);
+    ERR_remove_state(0);
     LogHandler_setPrefix(NULL);
     return SMFIS_TEMPFAIL;
 }
 
 
-/**
- * create loopback-address
- *
- * @param sa_family
- * @return
- */
-static _SOCK_ADDR *
-loopbackaddrdup(sa_family_t sa_family)
-{
-    switch (sa_family) {
-    case AF_INET:
-        {
-            // create IPv4 loopback-address
-            struct sockaddr_in *psock = (struct sockaddr_in *) malloc(sizeof(struct sockaddr_in));
-            if (NULL == psock) {
-                return NULL;
-            }
-            psock->sin_family = AF_INET;
-            psock->sin_addr.s_addr = INADDR_LOOPBACK;
-            psock->sin_port = htons(0);
-            return (_SOCK_ADDR *) psock;
-        }
-    case AF_INET6:
-        {
-            // create IPv6 loopback-address
-            struct sockaddr_in6 *psock =
-                (struct sockaddr_in6 *) malloc(sizeof(struct sockaddr_in6));
-            if (NULL == psock) {
-                return NULL;
-            }
-            psock->sin6_family = AF_INET6;
-            psock->sin6_addr = in6addr_loopback;
-            psock->sin6_port = htons(0);
-            return (_SOCK_ADDR *) psock;
-        }
-    default:
-        LogError("unknown sa_family: sa_family=%d", sa_family);
-        abort();
-    }
-}
-
-
 /**
  * duplicate hostaddr
  *
@@ -231,7 +192,7 @@ hostaddrdup(const _SOCK_ADDR *hostaddr)
         memcpy(psock, hostaddr, address_len);
         return psock;
     } else {
-        _SOCK_ADDR *psock = loopbackaddrdup(AF_INET6);
+        _SOCK_ADDR *psock = (_SOCK_ADDR *) loopbackAddrDup(AF_INET6);
         if (NULL == psock) {
             return NULL;
         }
@@ -268,26 +229,22 @@ ipaddrdup(const char *hostname, const _SOCK_ADDR *hostaddr)
     char addr_buf[INET6_ADDRSTRLEN];
 
     switch (hostaddr->sa_family) {
-    case AF_INET:
-        {
-            struct sockaddr_in *sin = (struct sockaddr_in *) hostaddr;
-            if (NULL == inet_ntop(AF_INET, &(sin->sin_addr), addr_buf, INET_ADDRSTRLEN)) {
-                LogError("inet_ntop AF_INET4 failed: hostname=%s, error=%s", NNSTR(hostname),
-                         strerror(errno));
-                return NULL;
-            }
-            break;
+    case AF_INET:;
+        struct sockaddr_in *sin4 = (struct sockaddr_in *) hostaddr;
+        if (NULL == inet_ntop(AF_INET, &(sin4->sin_addr), addr_buf, INET_ADDRSTRLEN)) {
+            LogError("inet_ntop AF_INET4 failed: hostname=%s, error=%s", NNSTR(hostname),
+                     strerror(errno));
+            return NULL;
         }
-    case AF_INET6:
-        {
-            struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) hostaddr;
-            if (NULL == inet_ntop(AF_INET6, &(sin6->sin6_addr), addr_buf, INET6_ADDRSTRLEN)) {
-                LogError("inet_ntop AF_INET6 failed: hostname=%s, error=%s", NNSTR(hostname),
-                         strerror(errno));
-                return NULL;
-            }
-            break;
+        break;
+    case AF_INET6:;
+        struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) hostaddr;
+        if (NULL == inet_ntop(AF_INET6, &(sin6->sin6_addr), addr_buf, INET6_ADDRSTRLEN)) {
+            LogError("inet_ntop AF_INET6 failed: hostname=%s, error=%s", NNSTR(hostname),
+                     strerror(errno));
+            return NULL;
         }
+        break;
     default:
         LogError("Unknown protocol: hostname=%s, sa_familyr=%d", NNSTR(hostname),
                  hostaddr->sa_family);
@@ -330,6 +287,29 @@ EnmaMfi_set_qid(SMFICTX *ctx, EnmaMfiCtx *enma_mfi_ctx)
     return true;
 }
 
+/**
+ * 接続元が、設定されているIPアドレスレンジであるかの判定
+ *
+ * @param enma_mfi_ctx
+ */
+static bool
+EnmaMfi_match_exclusion_address_ranges(EnmaMfiCtx *enma_mfi_ctx)
+{
+    char buf[INET6_ADDRSTRLEN];
+
+    const struct sockaddr *sa = (struct sockaddr *) enma_mfi_ctx->hostaddr;
+    if (!addrToIpStr(sa, buf)) {
+        LogError("addrToIpStr failed: error=%s", strerror(errno));
+    }
+    LogDebug("connected ip address=%s", buf);
+
+    if (IPAddressRangeList_matchToSocket(g_enma_config->common_exclusion_addresses, sa)) {
+        LogInfo("excluded ip address=%s", buf);
+        return true;
+    }
+
+    return false;
+}
 
 /**
  * Handler of each SMTP connection
@@ -372,6 +352,12 @@ mfi_connect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *hostaddr)
         return EnmaMfi_tempfail(enma_mfi_ctx);
     }
 
+    if (NULL != g_enma_config->common_exclusion_addresses
+        && EnmaMfi_match_exclusion_address_ranges(enma_mfi_ctx)) {
+        // 以降の処理をスキップ
+        return SMFIS_ACCEPT;
+    }
+
     return SMFIS_CONTINUE;
 }
 
@@ -496,8 +482,8 @@ mfi_header(SMFICTX *ctx, char *headerf, char *headerv)
             LogDebug("fraud AuthResultHeader: [No.%d] %s", enma_mfi_ctx->authhdr_count, headerv);
         }
     }
-    // SIDFが有効の場合ヘッダを格納する
-    if (g_enma_config->sidf_auth) {
+    // SIDF/DKIM が有効の場合ヘッダを格納する
+    if (g_enma_config->sidf_auth || g_enma_config->dkim_auth) {
         int pos = MailHeaders_append(enma_mfi_ctx->headers, headerf, headerv);
         if (pos < 0) {
             LogError("MailHeaders_append failed: headerf=%s, headerv=%s", NNSTR(headerf),
@@ -510,6 +496,73 @@ mfi_header(SMFICTX *ctx, char *headerf, char *headerv)
 }
 
 
+sfsistat
+mfi_eoh(SMFICTX *ctx)
+{
+    LogDebug("eoh");
+
+    EnmaMfiCtx *enma_mfi_ctx = smfi_getpriv(ctx);
+    if (NULL == enma_mfi_ctx) {
+        LogError("smfi_getpriv failed");
+        return SMFIS_TEMPFAIL;
+    }
+
+    if (g_enma_config->dkim_auth) {
+        // DkimVerifySession オブジェクトの初期化
+        enma_mfi_ctx->dkimverifier = DkimVerifySession_new(g_dkim_vpolicy, enma_mfi_ctx->resolver);
+        if (NULL == enma_mfi_ctx->dkimverifier) {
+            LogError("DkimVerifySession_new failed: err=%s", strerror(errno));
+            return EnmaMfi_tempfail(enma_mfi_ctx);
+        }   // end if
+        dkim_stat_t set_stat =
+            DkimVerifySession_setHeaders(enma_mfi_ctx->dkimverifier, enma_mfi_ctx->headers);
+        if (DSTAT_PERMFAIL_AUTHOR_AMBIGUOUS == set_stat) {
+            LogEvent("DKIM-skip", "Author ambiguous and verification is skipped.");
+        } else if (DSTAT_ISCRITERR(set_stat)) {
+            // エラーが発生した場合
+            LogError("DkimVerifySession_setHeaders failed: err=%s", DKIM_strerror(set_stat));
+            return EnmaMfi_tempfail(enma_mfi_ctx);
+        }   // end if
+        dkim_stat_t eoh_stat = DkimVerifySession_startBody(enma_mfi_ctx->dkimverifier);
+        if (DSTAT_VERIFIED_NO_SIGNHEADER == eoh_stat) {
+            // DKIM 署名ヘッダが1つもついていなかった場合
+            LogEvent("DKIM-skip", "No DKIM-Signature header found and verification is skipped.");
+        } else if (DSTAT_ISCRITERR(eoh_stat)) {
+            // エラーが発生した場合
+            LogError("DkimVerifySession_startBody failed: err=%s", DKIM_strerror(eoh_stat));
+            return EnmaMfi_tempfail(enma_mfi_ctx);
+        }   // end if
+    }
+
+    return SMFIS_CONTINUE;
+}
+
+
+sfsistat
+mfi_body(SMFICTX *ctx, unsigned char *bodyp, size_t bodylen)
+{
+    LogDebug("body");
+
+    EnmaMfiCtx *enma_mfi_ctx = smfi_getpriv(ctx);
+    if (NULL == enma_mfi_ctx) {
+        LogError("smfi_getpriv failed");
+        return SMFIS_TEMPFAIL;
+    }
+
+    if (g_enma_config->dkim_auth) {
+        dkim_stat_t body_stat =
+            DkimVerifySession_updateBody(enma_mfi_ctx->dkimverifier, bodyp, bodylen);
+        if (DSTAT_ISCRITERR(body_stat)) {
+            // エラーが発生した場合
+            LogError("DkimVerifySession_body failed: err=%s", DKIM_strerror(body_stat));
+            return EnmaMfi_tempfail(enma_mfi_ctx);
+        }   // end if
+    }   // end if
+
+    return SMFIS_CONTINUE;
+}
+
+
 /**
  * eom時に呼ばれるコールバック関数
  *
@@ -544,7 +597,7 @@ mfi_eom(SMFICTX *ctx)
 
     // Authentication-Results ヘッダの準備
     bool appended_stat =
-        AuthResult_appendAuthServer(enma_mfi_ctx->authresult, g_enma_config->authresult_identifier);
+        AuthResult_appendAuthServId(enma_mfi_ctx->authresult, g_enma_config->authresult_identifier);
     if (!appended_stat) {
         return EnmaMfi_tempfail(enma_mfi_ctx);
     }
@@ -556,6 +609,16 @@ mfi_eom(SMFICTX *ctx)
     if (g_enma_config->sidf_auth && !EnmaMfi_sidf_eom(enma_mfi_ctx, SIDF_RECORD_SCOPE_SPF2_PRA)) {
         return EnmaMfi_tempfail(enma_mfi_ctx);
     }
+    // DKIM
+    if (g_enma_config->dkim_auth
+        && !EnmaDkim_evaluate(enma_mfi_ctx->dkimverifier, enma_mfi_ctx->authresult)) {
+        return EnmaMfi_tempfail(enma_mfi_ctx);
+    }
+    // DKIM ADSP
+    if (g_enma_config->dkim_auth && g_enma_config->dkimadsp_auth
+        && !EnmaDkimAdsp_evaluate(enma_mfi_ctx->dkimverifier, enma_mfi_ctx->authresult)) {
+        return EnmaMfi_tempfail(enma_mfi_ctx);
+    }
     // Authentication-Results ヘッダをメッセージの先頭に挿入
     if (EOK != AuthResult_status(enma_mfi_ctx->authresult)) {
         LogError("AuthResult_status failed");
@@ -569,6 +632,7 @@ mfi_eom(SMFICTX *ctx)
     }
 
     EnmaMfiCtx_reset(enma_mfi_ctx);
+    ERR_remove_state(0);
     (void) LogHandler_setPrefix(NULL);
 
     return SMFIS_CONTINUE;
@@ -590,6 +654,7 @@ mfi_abort(SMFICTX *ctx)
     if (NULL != enma_mfi_ctx) {
         EnmaMfiCtx_reset(enma_mfi_ctx);
     }
+    ERR_remove_state(0);
     LogHandler_setPrefix(NULL);
 
     return SMFIS_CONTINUE;
